/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <unistd.h>
#include <stdio.h>

#include "getopts.h"

/* True/False macros. */
#define FALSE			(0)
#define TRUE			(!FALSE)

/* External definitions. */
extern char	*argv0;

/* Flags and options. */
static int	gflag	= FALSE;
static int	pflag	= FALSE;
static int	Sflag	= FALSE;
static int	Pflag	= FALSE;
static int	Oflag	= FALSE;
static int	Mflag	= FALSE;


/*
 * usage()
 *
 * Print usage message, and exit.
 */
static void
usage (void)
{
	fprintf (stderr, "Try '%s -h' for more information.\n", argv0);
	exit (1);
}


/*
 * help()
 *
 * Print help message, and exit.
 */
static void
help (void)
{
	fprintf (stderr, "\
Usage: %s [ options ] file [ file ... ]\n\
       %s -M id [ id ... ]\n\n\
Options:\n\
	-g	Include debugging information in the output files\n\
	-p	Include line profiling for gmon.out execution profile\n\
	-O	Optimize generated code (minimal optimization only)\n\
	-P	Write out intermediate file (.p) during compilation\n\
	-S,-s	Write out assembly language file (.s) during compilation\n\
	-M	Name-mangle each id given, for linking with 'C'\n\
	-v	Print compiler version number and exit\n\
	-h	Print this help message\n",
		argv0, argv0);
	exit (0);
}


/*
 * version()
 *
 * Print version message, and exit.
 */
static void
version (void)
{
	printf ("\
\n%s version %s %s\n\
\t\tCopyright (C) 2001  Simon Baldwin\n\n\
This program comes with ABSOLUTELY NO WARRANTY; for details\n\
please see the file 'COPYING' supplied with the source code.\n\
This is free software, and you are welcome to redistribute it\n\
under certain conditions; again, see 'COPYING' for details.\n\
This program is released under the GNU General Public License.\n",
argv0, __DATE__, __TIME__);
	exit (0);
}


/*
 * parse_cmdline()
 *
 * Parse command line arguments into options and flags.
 */
int
parse_cmdline (int argc, char *argv[])
{
	int	c;				/* Option character */

	/* Process all options. */
	while ((c = getopt (argc, argv, "gpOPSsMvh")) != -1)
	    {
		switch (c)
		    {
			case 'g': gflag = TRUE;	break;
			case 'p': pflag = TRUE;	break;
			case 'O': Oflag = TRUE;	break;
			case 'P': Pflag = TRUE;	break;
			case 'S': Sflag = TRUE;	break;
			case 's': Sflag = TRUE;	break;
			case 'M': Mflag = TRUE;	break;
			case 'v': version ();	break;
			case 'h': help ();	break;
			case '?': usage ();	break;
		    }
	}

	/* There must be at least one argument after options. */
	if (optind >= argc)
	    {
		if (!Mflag)
			fprintf (stderr, "%s: no input files\n", argv0);
		else
			fprintf (stderr, "%s: no ids given\n", argv0);
		usage ();
	    }

	/* Check combinations. */
	if (gflag && pflag)
	    {
		fprintf (stderr,
			 "%s: -g flag overrides -p flag; -p ignored\n",
			 argv0);
		pflag = FALSE;
	    }
	if (Mflag && (gflag || pflag || Oflag || Pflag || Sflag))
	    {
		fprintf (stderr,
			 "%s: -M cannot be used with other flags; -M ignored\n",
			 argv0);
		Mflag = FALSE;
	    }

	/* return the index to the first non-option */
	return optind;
}


/*
 * get_*flag()
 *
 * Return option flag values to caller.
 */
int
get_gflag (void)
{
	return gflag;
}
int
get_pflag (void)
{
	return pflag;
}
int
get_Oflag (void)
{
	return Oflag;
}
int
get_Pflag (void)
{
	return Pflag;
}
int
get_Sflag (void)
{
	return Sflag;
}
int
get_Mflag (void)
{
	return Mflag;
}
