/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/param.h>
#include <stdarg.h>
#include <errno.h>

#include "sfile.h"

/* File stream and corresponding file name. */
static FILE	*fs			= NULL;
static char	fname[MAXPATHLEN]	= "";


/*
 * close_sfile()
 *
 * Close the current sfile.
 */
void
close_sfile (void)
{
	/* Do nothing if no file is open. */
	if (fs != NULL)
	    {
		/* Flush, then close file, and reset pointers. */
		fflush (fs);
		fclose (fs);
		fs = NULL;
	    }
}


/*
 * open_sfile()
 *
 * Open a new sfile.
 */
int
open_sfile (const char *path)
{
	assert (path != NULL);

	/* Close any already-open file, then open the new one. */
	close_sfile ();
	fs = fopen (path, "w");
	if (fs == NULL)
		return errno;

	/* Note the path to the opened file. */
	strcpy (fname, path);
	return 0;
}


/*
 * unlink_sfile()
 *
 * Delete an sfile, on parse error.
 */
void
unlink_sfile (void)
{
	/* Close the file if open, then unlink it. */
	close_sfile ();
	if (strlen (fname) != 0)
	    {
		unlink (fname);
		strcpy (fname, "");
	    }
}


/*
 * printf_sfile()
 *
 * Printf function specific to the current open sfile.
 */
void
printf_sfile (const char *format, ...)
{
	va_list		ap;			/* Varargs list. */
	assert (format != NULL);

	/* Find the printf arguments after 'format'. */
	va_start (ap, format);

	/* Pass on the data to vfprintf as we receive it. */
	vfprintf (fs, format, ap);

	/* Done with varargs. */
	va_end (ap);
}
