/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <assert.h>
#include <unistd.h>
#include <string.h>

#include "utils.h"

/*
 * basefilename()
 *
 * Return the basename of a file path.
 */
char *
basefilename (char *path)
{
	int	pp;				/* Index into path */
	assert (path != NULL);

	/* Back up through the string to the first '/' character. */
	for (pp = strlen (path) - 1; path[pp] != '/' && pp > 0;)
		pp--;

	/* If we found a '/', move forward over it again. */
	if (path[pp] == '/')
		pp++;

	/* Return the address of the base part of the filename. */
	return path + pp;
}


/*
 * stripext()
 *
 * Strip the extension from a file path.
 */
void
stripext (char *path)
{
	int	pp;				/* Index into path */
	assert (path != NULL);

	/* Back up through the string until we find '.', or '/'. */
	for (pp = strlen (path) - 1;
			path[pp] != '.' && path[pp] != '/' && pp >= 0;)
		pp--;

	/* If we found '.', replace it with NUL, and we are done. */
	if (path[pp] == '.')
		path[pp] = '\0';
}
